# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging

import pytest
# Allow everything in there to access the DB
pytestmark = pytest.mark.django_db

from django.db import IntegrityError
from django.db.models import ProtectedError
from django.core.exceptions import ValidationError as DjangoValidationError
import ipaddress

from nsot import exc, models

from .fixtures import admin_user, user, site, transactional_db
from ..util import load_json


def test_reparent_bug_issues_27(site):
    """
    Test for bug described at https://github.com/dropbox/nsot/issues/27
    """

    net_8  = models.Network.objects.create(site=site, cidr=u"10.0.0.0/8")
    net_31 = models.Network.objects.create(site=site, cidr=u"10.17.244.128/31")
    net_25 = models.Network.objects.create(site=site, cidr=u"10.16.1.0/25")

    net_8.refresh_from_db()
    net_31.refresh_from_db()
    net_25.refresh_from_db()

    assert net_8.parent_id is None
    assert net_31.parent_id == net_8.id
    assert net_25.parent_id == net_8.id


def test_next_network_bug_issues_216(site):
    """
    Test for bug where Network.get_next_network() returns the wrong result.

    Ref: https://github.com/dropbox/nsot/issues/216
    """

    # Load and create the Network objects
    objects = load_json('model_tests/data/networks.json')
    [models.Network.objects.create(site=site, **n) for n in objects]

    # Get the parent
    parent = models.Network.objects.get_by_address('10.20.0.0/16')

    # We're expecting that the next /21 is going to be expected_cidr
    expected_cidr = '10.20.192.0/21'
    expected = [ipaddress.ip_network(expected_cidr)]
    next_21 = parent.get_next_network(prefix_length=21)

    assert next_21 == expected

    # pytest.set_trace()



import os


def pytest_report_header(config):
    """Customize the report header to display API version."""
    api_version = os.getenv('NSOT_API_VERSION')
    return 'Using NSoT API version: %s' % api_version



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import smart_selects.db_fields
import django.db.models.deletion
import django_extensions.db.fields.json
import nsot.fields


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0014_auto_20151002_1653'),
    ]

    operations = [

        ##########
        # Device #
        ##########
        # Create temp Device object
        migrations.CreateModel(
            name='Device_temp',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('_attributes_cache', django_extensions.db.fields.json.JSONField(blank=True)),
                ('hostname', models.CharField(max_length=255, db_index=True)),
                ('site', models.ForeignKey(related_name='devices', on_delete=django.db.models.deletion.PROTECT, to='nsot.Site')),
            ],
        ),

        # self._attributes_cache = self.resource_ptr._attributes
        migrations.AddField(
            model_name='device',
            name='_attributes_cache',
            field=django_extensions.db.fields.json.JSONField(blank=True),
        ),

        # self.id = self.resource_ptr_id
        migrations.AddField(
            model_name='device',
            name='new_id',
            field=models.IntegerField(default=0, verbose_name='ID'),
            preserve_default=False,
        ),

        ###########
        # Network #
        ###########
        migrations.CreateModel(
            name='Network_temp',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('_attributes_cache', django_extensions.db.fields.json.JSONField(blank=True)),
                ('network_address', nsot.fields.BinaryIPAddressField(max_length=16, db_index=True)),
                ('broadcast_address', nsot.fields.BinaryIPAddressField(max_length=16, db_index=True)),
                ('prefix_length', models.IntegerField(db_index=True)),
                ('ip_version', models.CharField(db_index=True, max_length=1, choices=[('4', '4'), ('6', '6')])),
                ('is_ip', models.BooleanField(default=False, db_index=True)),
                ('parent', models.ForeignKey(related_name='children', on_delete=django.db.models.deletion.PROTECT, default=None, blank=True, to='nsot.Network_temp', null=True),
        ),
                ('site', models.ForeignKey(related_name='networks', on_delete=django.db.models.deletion.PROTECT, to='nsot.Site')),
            ],
        ),

        migrations.AddField(
            model_name='network',
            name='_attributes_cache',
            field=django_extensions.db.fields.json.JSONField(blank=True),
        ),
        migrations.AddField(
            model_name='network',
            name='new_id',
            field=models.IntegerField(default=0, verbose_name='ID'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='network',
            name='new_parent_id',
            field=models.IntegerField(null=True, verbose_name='Parent ID'),
            preserve_default=False,
        ),

        ##################
        # Assignment 1/2 #
        ##################

        migrations.CreateModel(
            name='Assignment_temp',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('address', models.ForeignKey(to='nsot.Network_temp')),
                # ('interface', models.ForeignKey(to='nsot.Interface_temp')),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),

        #############
        # Interface #
        #############
        migrations.CreateModel(
            name='Interface_temp',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('_attributes_cache', django_extensions.db.fields.json.JSONField(blank=True)),
                ('name', models.CharField(help_text='The name of the interface as it appears on the Device.', max_length=255, db_index=True)),
                ('description', models.CharField(default='', help_text='A brief yet helpful description.', max_length=255, blank=True)),
                ('type', models.IntegerField(default=6, help_text="If not provided, defaults to 'ethernet'.", db_index=True, verbose_name='Interface Type', choices=[(6, b'ethernet'), (1, b'other'), (135, b'l2vlan'), (136, b'l3vlan'), (161, b'lag'), (24, b'loopback'), (150, b'mpls'), (53, b'prop_virtual'), (131, b'tunnel')])),
                ('mac_address', nsot.fields.MACAddressField(default=0, blank=True, help_text='If not provided, defaults to 00:00:00:00:00:00.', integer=True, null=True, verbose_name='MAC Address', db_index=True)),
                ('speed', models.IntegerField(default=10000, help_text='Integer of Mbps of interface (e.g. 20000 for 20 Gbps). If not provided, defaults to 10000.', db_index=True, blank=True)),
                ('addresses', models.ManyToManyField(related_name='addresses', through='nsot.Assignment_temp', to='nsot.Network_temp', db_index=True)),
                ('device', models.ForeignKey(related_name='interfaces', verbose_name='Device', to='nsot.Device_temp', help_text='Unique ID of the connected Device.')),
                ('parent', smart_selects.db_fields.ChainedForeignKey(chained_model_field='device', related_name='children', on_delete=django.db.models.deletion.PROTECT, default=None, auto_choose=True, to='nsot.Interface_temp', chained_field='device', blank=True, help_text='Unique ID of the parent Interface.', null=True, verbose_name='Parent')),
                ('site', models.ForeignKey(related_name='interfaces', on_delete=django.db.models.deletion.PROTECT, to='nsot.Site')),
            ],
        ),

        migrations.AddField(
            model_name='interface',
            name='_attributes_cache',
            field=django_extensions.db.fields.json.JSONField(blank=True),
        ),
        migrations.AddField(
            model_name='interface',
            name='new_id',
            field=models.IntegerField(default=0, verbose_name='ID'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='interface',
            name='new_parent_id',
            field=models.IntegerField(null=True, verbose_name='Parent ID'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='device',
            name='new_device_id',
            field=models.IntegerField(default=0, verbose_name='Device ID'),
            preserve_default=False,
        ),

        ##################
        # Assignment 2/2 # 
        ##################
        migrations.AddField(
            model_name='assignment_temp',
            name='interface',
            field=models.ForeignKey(to='nsot.Interface_temp'),
        ),
        migrations.AlterUniqueTogether(
            name='interface_temp',
            unique_together=set([('device', 'name')]),
        ),
        migrations.AlterIndexTogether(
            name='interface_temp',
            index_together=set([('device', 'name')]),
        ),
        migrations.AlterUniqueTogether(
            name='assignment_temp',
            unique_together=set([('address', 'interface')]),
        ),
        migrations.AlterIndexTogether(
            name='assignment_temp',
            index_together=set([('address', 'interface')]),
        ),

        #########
        # Value #
        #########

        # self.attribute.name => self.name
        migrations.AddField(
            model_name='value',
            name='name',
            field=models.CharField(max_length=64, verbose_name='Name', blank=True),
        ),

        # self.resource_id => self.new_resource_id
        migrations.AddField(
            model_name='value',
            name='new_resource_id',
            field=models.IntegerField(default=1, verbose_name='Resource ID'),
            preserve_default=False,
        ),

        # self.resource_name = self.resource.__class__.__name__
        migrations.AddField(
            model_name='value',
            name='resource_name',
            field=models.CharField(default='BOGUS', max_length=20, verbose_name='Resource Type', db_index=True, choices=[('Device', 'Device'), ('Attribute', 'Attribute'), ('Interface', 'Interface'), ('Site', 'Site'), ('Network', 'Network')]),
            preserve_default=False,
        ),

        ##########
        # Change #
        ##########

        migrations.AlterField(
            model_name='change',
            name='resource_name',
            field=models.CharField(db_index=True, max_length=20, verbose_name='Resource Type', choices=[('Device', 'Device'), ('Attribute', 'Attribute'), ('Interface', 'Interface'), ('Site', 'Site'), ('Network', 'Network')]),
        ),

]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_extensions.db.fields.json


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0021_remove_resource_object'),
    ]

    operations = [
        migrations.AddField(
            model_name='interface',
            name='_addresses_cache',
            field=django_extensions.db.fields.json.JSONField(default=b'[]', blank=True),
        ),
        migrations.AddField(
            model_name='interface',
            name='_networks_cache',
            field=django_extensions.db.fields.json.JSONField(default=b'[]', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import macaddress.fields


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Assignment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('address', models.ForeignKey(to='nsot.Network')),
            ],
        ),
        migrations.CreateModel(
            name='Interface',
            fields=[
                ('resource_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='nsot.Resource')),
                ('name', models.CharField(max_length=255, db_index=True)),
                ('description', models.CharField(default='', max_length=255)),
                ('type', models.IntegerField(db_index=True, verbose_name='Interface Type', choices=[('other', 1), ('ethernet', 6), ('loopback', 24), ('tunnel', 131), ('l2vlan', 135), ('l3vlan', 136), ('mpls', 150), ('lag', 161)])),
                ('mac', macaddress.fields.MACAddressField(integer=True, db_index=True, blank=True)),
                ('speed', models.IntegerField(db_index=True)),
                ('addresses', models.ManyToManyField(related_name='addresses', through='nsot.Assignment', to='nsot.Network', db_index=True)),
                ('device', models.ForeignKey(related_name='interfaces', to='nsot.Device')),
            ],
            bases=('nsot.resource',),
        ),
        migrations.AddField(
            model_name='assignment',
            name='interface',
            field=models.ForeignKey(to='nsot.Interface'),
        ),
        migrations.AlterUniqueTogether(
            name='interface',
            unique_together=set([('device', 'name')]),
        ),
        migrations.AlterIndexTogether(
            name='interface',
            index_together=set([('device', 'name')]),
        ),
        migrations.AlterUniqueTogether(
            name='assignment',
            unique_together=set([('address', 'interface')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import nsot.fields


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0009_auto_20150811_1245'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='description',
            field=models.CharField(default='', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='description',
            field=models.CharField(default='', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='mac_address',
            field=nsot.fields.MACAddressField(default=0, blank=True, help_text='If not provided, defaults to 00:00:00:00:00:00.', integer=True, null=True, verbose_name='MAC Address', db_index=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='name',
            field=models.CharField(help_text='The name of the interface as it appears on the device.', max_length=255, db_index=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='speed',
            field=models.IntegerField(default=10000, help_text='Integer of Mbps of interface (e.g. 20000 for 20 Gbps). If not provided, defaults to 10000.', db_index=True, blank=True),
        ),
        migrations.AlterField(
            model_name='interface',
            name='type',
            field=models.IntegerField(default=6, help_text="If not provided, defaults to 'ethernet'.", db_index=True, verbose_name='Interface Type', choices=[(6, b'ethernet'), (1, b'other'), (135, b'l2vlan'), (136, b'l3vlan'), (161, b'lag'), (24, b'loopback'), (150, b'mpls'), (131, b'tunnel')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0008_auto_20150811_1222'),
    ]

    operations = [
        migrations.AlterField(
            model_name='interface',
            name='type',
            field=models.IntegerField(default=6, db_index=True, verbose_name='Interface Type', choices=[(1, 'other'), (6, 'ethernet'), (24, 'loopback'), (131, 'tunnel'), (135, 'l2vlan'), (136, 'l3vlan'), (150, 'mpls'), (161, 'lag')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0007_auto_20150811_1201'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assignment',
            name='address',
            field=models.ForeignKey(related_name='assignments', to='nsot.Network'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0012_auto_20151002_1427'),
    ]

    operations = [
        migrations.AlterField(
            model_name='interface',
            name='name',
            field=models.CharField(help_text='The name of the interface as it appears on the Device.', max_length=255, db_index=True),
        ),
    ]



#!/usr/bin/env python

from distutils import log
from distutils.core import Command
import os
from subprocess import check_output
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
from setuptools.command.develop import develop as DevelopCommand
from setuptools.command.sdist import sdist as SDistCommand


ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__)))

execfile('nsot/version.py')

with open('requirements.txt') as requirements:
    required = requirements.read().splitlines()


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', 'Arguments to pass to py.test')]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # Import here, cause outside the eggs aren't loaded
        log.info('Running python tests...')
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


class DevelopWithBuildStatic(DevelopCommand):
    def install_for_development(self):
        self.run_command('build_static')
        return DevelopCommand.install_for_development(self)


class SDistWithBuildStatic(SDistCommand):
    def run(self):
        self.run_command('build_static')
        SDistCommand.run(self)


class BuildStatic(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        log.info('running [npm install --quiet]')
        check_output(['npm', 'install', '--quiet'], cwd=ROOT)

        log.info('running [gulp clean]')
        check_output(
            [os.path.join(ROOT, 'node_modules', '.bin', 'gulp'), 'clean'],
            cwd=ROOT
        )

        log.info('running [gulp build]')
        check_output(
            [os.path.join(ROOT, 'node_modules', '.bin', 'gulp'), 'build'],
            cwd=ROOT
        )


kwargs = {
    'name': 'nsot',
    'version': str(__version__),  # noqa
    'packages': find_packages(exclude=['tests*']),
    'include_package_data': True,
    'description': 'Network Source of Truth (IP Address Management).',
    'author': 'Gary M. Josack',
    'maintainer': 'Gary M. Josack',
    'author_email': 'gary@dropbox.com',
    'maintainer_email': 'gary@dropbox.com',
    'license': 'Apache',
    'install_requires': required,
    'url': 'https://github.com/dropbox/nsot',
    'tests_require': ['pytest'],
    'cmdclass': {
        'test': PyTest,
        'build_static': BuildStatic,
        'develop': DevelopWithBuildStatic,
        'sdist': SDistWithBuildStatic,
    },
    'entry_points': """
        [console_scripts]
        nsot-server=nsot.util:main
        snot-server=nsot.util:main
    """,
    'classifiers': [
        'Programming Language :: Python',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
}

setup(**kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nsot', '0005_auto_20150810_1847'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assignment',
            name='interface',
            field=models.ForeignKey(related_name='assignments', to='nsot.Interface'),
        ),
        migrations.AlterIndexTogether(
            name='assignment',
            index_together=set([('address', 'interface')]),
        ),
    ]
